exc_type  # unused variable (/data/repos/redis/redis-py/redis/client.py:1045)
exc_value  # unused variable (/data/repos/redis/redis-py/redis/client.py:1045)
traceback  # unused variable (/data/repos/redis/redis-py/redis/client.py:1045)
exc_type  # unused variable (/data/repos/redis/redis-py/redis/client.py:1211)
exc_value  # unused variable (/data/repos/redis/redis-py/redis/client.py:1211)
traceback  # unused variable (/data/repos/redis/redis-py/redis/client.py:1211)
exc_type  # unused variable (/data/repos/redis/redis-py/redis/client.py:1589)
exc_value  # unused variable (/data/repos/redis/redis-py/redis/client.py:1589)
traceback  # unused variable (/data/repos/redis/redis-py/redis/client.py:1589)
exc_type  # unused variable (/data/repos/redis/redis-py/redis/lock.py:156)
exc_value  # unused variable (/data/repos/redis/redis-py/redis/lock.py:156)
traceback  # unused variable (/data/repos/redis/redis-py/redis/lock.py:156)
exc_type  # unused variable (/data/repos/redis/redis-py/redis/asyncio/utils.py:26)
exc_value  # unused variable (/data/repos/redis/redis-py/redis/asyncio/utils.py:26)
traceback  # unused variable (/data/repos/redis/redis-py/redis/asyncio/utils.py:26)
AsyncConnectionPool  # unused import (//data/repos/redis/redis-py/redis/typing.py:9)
AsyncRedis  # unused import (//data/repos/redis/redis-py/redis/commands/core.py:49)
TargetNodesT  # unused import (//data/repos/redis/redis-py/redis/commands/cluster.py:46)
